@@ -66,7 +66,52 @@ Author: Ralph Mor, X Consortium
     IceDisposeCompleteMessage (_iceConn, _pStart);\
     return (0);\
 }
-
+
+#ifndef HAVE_ASPRINTF
+# include <stdarg.h>
+
+/* sprintf variant found in newer libc's which allocates string to print to */
+static int _X_ATTRIBUTE_PRINTF(2,3)
+asprintf(char ** ret, const char *format, ...)
+{
+    char buf[256];
+    int len;
+    va_list ap;
+
+    va_start(ap, format);
+    len = vsnprintf(buf, sizeof(buf), format, ap);
+    va_end(ap);
+
+    if (len < 0)
+	return -1;
+
+    if (len < sizeof(buf))
+    {
+	*ret = strdup(buf);
+    }
+    else
+    {
+	*ret = malloc(len + 1); /* snprintf doesn't count trailing '\0' */
+	if (*ret != NULL)
+	{
+	    va_start(ap, format);
+	    len = vsnprintf(*ret, len + 1, format, ap);
+	    va_end(ap);
+	    if (len < 0) {
+		free(*ret);
+		*ret = NULL;
+	    }
+	}
+    }
+
+    if (*ret == NULL)
+	return -1;
+
+    return len;
+}
+#endif
+
+
 /*
  * IceProcessMessages:
  *
@@ -259,7 +304,7 @@ IceProcessMessages (
 	 * to use message replies unless we translate opcodes before
 	 * comparing them.
 	 */
-	
+
 	{
 	    int op;
 
@@ -385,7 +430,7 @@ IceProcessMessages (
 }
 
 
-
+
 static void
 AuthRequired (
 	IceConn		iceConn,
@@ -412,7 +457,7 @@ AuthRequired (
 }
 
 
-
+
 static void
 AuthReply (
 	IceConn		iceConn,
@@ -437,7 +482,7 @@ AuthReply (
 }
 
 
-
+
 static void
 AuthNextPhase (
 	IceConn		iceConn,
@@ -462,7 +507,7 @@ AuthNextPhase (
 }
 
 
-
+
 static void
 AcceptConnection (
 	IceConn iceConn,
@@ -490,7 +535,7 @@ AcceptConnection (
 }
 
 
-
+
 static void
 AcceptProtocol (
 	IceConn iceConn,
@@ -530,7 +575,7 @@ AcceptProtocol (
 }
 
 
-
+
 static void
 PingReply (
 	IceConn iceConn
@@ -541,7 +586,7 @@ PingReply (
 }
 
 
-
+
 static Bool
 ProcessError (
 	IceConn		 iceConn,
@@ -605,8 +650,8 @@ ProcessError (
 	    _IceConnectionError *errorReply =
 	        &(((_IceReply *) (replyWait->reply))->connection_error);
 	    char *errorStr = NULL;
-	    const char *tempstr;
-	    char *prefix, *temp;
+	    const char *tempstr, *prefix;
+	    char *temp;
 
 	    invokeHandler = 0;
 	    errorReturned = True;
@@ -632,9 +677,8 @@ ProcessError (
 		prefix = "Connection Setup Failed, reason : ";
 
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -642,9 +686,8 @@ ProcessError (
 
 		prefix = "Authentication Rejected, reason : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -652,9 +695,8 @@ ProcessError (
 
 		prefix = "Authentication Failed, reason : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -662,6 +704,11 @@ ProcessError (
 		invokeHandler = 1;
 	    }
 
+			if (!errorStr)
+			{
+				errorStr = strdup("");
+			}
+
 	    errorReply->type = ICE_CONNECTION_ERROR;
 	    errorReply->error_message = errorStr;
 	}
@@ -673,8 +720,9 @@ ProcessError (
 	{
 	    _IceProtocolError *errorReply =
 	        &(((_IceReply *) (replyWait->reply))->protocol_error);
-	    char *errorStr = "";
-	    char *prefix, *temp;
+	    char *errorStr = NULL;
+	    const char *prefix;
+	    char *temp;
 
 	    invokeHandler = 0;
 	    errorReturned = True;
@@ -683,16 +731,15 @@ ProcessError (
 	    {
 	    case IceNoVersion:
 
-	        temp =
-		    "None of the protocol versions specified are supported";
-		errorStr = strdup(temp);
+		errorStr = strdup(
+		    "None of the protocol versions specified are supported");
 		break;
 
 	    case IceNoAuth:
 
-		temp =
-		    "None of the authentication protocols specified are supported";
-		errorStr = strdup(temp);
+		errorStr = strdup(
+		    "None of the authentication protocols specified are supported");
+
 		break;
 
 	    case IceSetupFailed:
@@ -700,9 +747,8 @@ ProcessError (
 		prefix = "Protocol Setup Failed, reason : ";
 
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -710,9 +756,8 @@ ProcessError (
 
 		prefix = "Authentication Rejected, reason : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -720,9 +765,8 @@ ProcessError (
 
 		prefix = "Authentication Failed, reason : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -730,26 +774,24 @@ ProcessError (
 
 		prefix = "Protocol was already registered : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
 	    case IceMajorOpcodeDuplicate:
 
 		prefix = "The major opcode was already used : ";
-		errorStr = (char *) malloc (strlen (prefix) + 2);
-		sprintf (errorStr, "%s%d", prefix, (int) *pData);
+		if (asprintf (&errorStr, "%s%d", prefix, (int) *pData) == -1)
+		    errorStr = NULL;
 		break;
 
 	    case IceUnknownProtocol:
 
 		prefix = "Unknown Protocol : ";
 		EXTRACT_STRING (pData, swap, temp);
-		errorStr = (char *) malloc (
-		    strlen (prefix) + strlen (temp) + 1);
-		sprintf (errorStr, "%s%s", prefix, temp);
+		if (asprintf (&errorStr, "%s%s", prefix, temp) == -1)
+		    errorStr = NULL;
 		free (temp);
 		break;
 
@@ -757,6 +799,11 @@ ProcessError (
 		invokeHandler = 1;
 	    }
 
+			if (!errorStr)
+			{
+				errorStr = strdup("");
+			}
+
 	    errorReply->type = ICE_PROTOCOL_ERROR;
 	    errorReply->error_message = errorStr;
 	}
@@ -810,7 +857,7 @@ ProcessError (
 }
 
 
-
+
 static int
 ProcessConnectionSetup (
 	IceConn		iceConn,
@@ -819,12 +866,13 @@ ProcessConnectionSetup (
 )
 {
     iceConnectionSetupMsg *message;
-    int  myVersionCount, hisVersionCount;
+    const int myVersionCount = _IceVersionCount;
+    int  hisVersionCount;
     int	 myVersionIndex, hisVersionIndex;
     int  hisMajorVersion, hisMinorVersion;
     int	 myAuthCount, hisAuthCount;
     int	 found, i, j;
-    char *myAuthName, **hisAuthNames = NULL;
+    char **hisAuthNames = NULL;
     char *pData, *pStart, *pEnd;
     char *vendor = NULL;
     char *release = NULL;
@@ -850,17 +898,17 @@ ProcessConnectionSetup (
 
     pData = pStart;
     pEnd = pStart + (length << 3);
-    
-    SKIP_STRING (pData, swap, pEnd, 
+
+    SKIP_STRING (pData, swap, pEnd,
 		 BAIL_STRING(iceConn, ICE_ConnectionSetup,
 			     pStart));			       /* vendor */
-    SKIP_STRING (pData, swap, pEnd, 
+    SKIP_STRING (pData, swap, pEnd,
 		 BAIL_STRING(iceConn, ICE_ConnectionSetup,
 			    pStart));	        	       /* release */
-    SKIP_LISTOF_STRING (pData, swap, (int) message->authCount, pEnd, 
+    SKIP_LISTOF_STRING (pData, swap, (int) message->authCount, pEnd,
 			BAIL_STRING(iceConn, ICE_ConnectionSetup,
 				   pStart));		       /* auth names */
-    
+
     pData += (message->versionCount * 4);		       /* versions */
 
     CHECK_COMPLETE_SIZE (iceConn, ICE_ConnectionSetup,
@@ -882,14 +930,14 @@ ProcessConnectionSetup (
     EXTRACT_STRING (pData, swap, vendor);
     EXTRACT_STRING (pData, swap, release);
 
-    if ((hisAuthCount = message->authCount) > 0)
+    hisAuthCount = message->authCount;
+    if (hisAuthCount > 0)
     {
-	hisAuthNames = (char **) malloc (hisAuthCount * sizeof (char *));
+	hisAuthNames = malloc (hisAuthCount * sizeof (char *));
 	EXTRACT_LISTOF_STRING (pData, swap, hisAuthCount, hisAuthNames);
     }
 
     hisVersionCount = message->versionCount;
-    myVersionCount = _IceVersionCount;
 
     hisVersionIndex = myVersionIndex = found = 0;
 
@@ -922,8 +970,8 @@ ProcessConnectionSetup (
 	{
 	    for (i = 0; i < hisAuthCount; i++)
 		free (hisAuthNames[i]);
-	
-	    free ((char *) hisAuthNames);
+
+	    free (hisAuthNames);
 	}
 
 	IceDisposeCompleteMessage (iceConn, pStart);
@@ -946,7 +994,7 @@ ProcessConnectionSetup (
     {
 	if (authUsableFlags[i])
 	{
-	    myAuthName = _IceAuthNames[i];
+	    const char *myAuthName = _IceAuthNames[i];
 
 	    for (j = 0; j < hisAuthCount && !found; j++)
 		if (strcmp (myAuthName, hisAuthNames[j]) == 0)
@@ -981,7 +1029,7 @@ ProcessConnectionSetup (
 	    {
 		accept_setup_now = 1;
 	    }
-	    else 
+	    else
 	    {
 		_IceErrorAuthenticationRejected (iceConn,
 	            ICE_ConnectionSetup, "None of the authentication protocols specified are supported and host-based authentication failed");
@@ -989,8 +1037,7 @@ ProcessConnectionSetup (
 		iceConn->connection_status = IceConnectRejected;
 	    }
 
-	    if (hostname)
-		free (hostname);
+	    free (hostname);
 	}
 
 	if (iceConn->connection_status == IceConnectRejected)
@@ -1019,7 +1066,7 @@ ProcessConnectionSetup (
 
 	    AuthRequired (iceConn, hisAuthIndex, authDataLen, authData);
 
-	    iceConn->connect_to_me = setupInfo = (_IceConnectToMeInfo *)
+	    iceConn->connect_to_me = setupInfo =
 		malloc (sizeof (_IceConnectToMeInfo));
 
 	    setupInfo->my_version_index = myVersionIndex;
@@ -1034,14 +1081,18 @@ ProcessConnectionSetup (
 	{
 	    accept_setup_now = 1;
 	}
+	else
+	{
+	    free (vendor);
+	    free (release);
+	}
 
 	if (authData && authDataLen > 0)
-	    free ((char *) authData);
+	    free (authData);
 
-	if (errorString)
-	    free (errorString);
+	free (errorString);
     }
-    
+
     if (accept_setup_now)
     {
 	AcceptConnection (iceConn, hisVersionIndex);
@@ -1055,8 +1106,8 @@ ProcessConnectionSetup (
     {
 	for (i = 0; i < hisAuthCount; i++)
 	    free (hisAuthNames[i]);
-	
-	free ((char *) hisAuthNames);
+
+	free (hisAuthNames);
     }
 
     IceDisposeCompleteMessage (iceConn, pStart);
@@ -1064,7 +1115,7 @@ ProcessConnectionSetup (
 }
 
 
-
+
 static Bool
 ProcessAuthRequired (
 	IceConn			iceConn,
@@ -1209,7 +1260,8 @@ ProcessAuthRequired (
     }
     else if (status == IcePoAuthRejected || status == IcePoAuthFailed)
     {
-	char *prefix, *returnErrorString;
+	const char *prefix;
+	char *returnErrorString;
 
 	if (status == IcePoAuthRejected)
 	{
@@ -1226,11 +1278,10 @@ ProcessAuthRequired (
 	    prefix = "Authentication Failed, reason : ";
 	}
 
-	returnErrorString = (char *) malloc (strlen (prefix) +
-	    strlen (errorString) + 1);
-	sprintf (returnErrorString, "%s%s", prefix, errorString);
+	if (asprintf (&returnErrorString, "%s%s", prefix, errorString) == -1)
+	    returnErrorString = strdup("");
 	free (errorString);
-	
+
 	if (iceConn->connect_to_you)
 	{
 	    _IceConnectionError *errorReply =
@@ -1250,7 +1301,7 @@ ProcessAuthRequired (
     }
 
     if (replyData && replyDataLen > 0)
-	free ((char *) replyData);
+	free (replyData);
 
     IceDisposeCompleteMessage (iceConn, authData);
 
@@ -1258,7 +1309,7 @@ ProcessAuthRequired (
 }
 
 
-
+
 static int
 ProcessAuthReply (
 	IceConn		iceConn,
@@ -1327,15 +1378,14 @@ ProcessAuthReply (
 		    status = IcePaAuthAccepted;
 		}
 
-		if (hostname)
-		    free (hostname);
+		free (hostname);
 	    }
 
 	    if (status != IcePaAuthAccepted)
 	    {
 		free (iceConn->connect_to_me->his_vendor);
 		free (iceConn->connect_to_me->his_release);
-		free ((char *) iceConn->connect_to_me);
+		free (iceConn->connect_to_me);
 		iceConn->connect_to_me = NULL;
 
 		iceConn->connection_status = IceConnectRejected;
@@ -1363,7 +1413,7 @@ ProcessAuthReply (
 	    iceConn->my_ice_version_index =
 		iceConn->connect_to_me->my_version_index;
 
-	    free ((char *) iceConn->connect_to_me);
+	    free (iceConn->connect_to_me);
 	    iceConn->connect_to_me = NULL;
 	}
     }
@@ -1402,8 +1452,7 @@ ProcessAuthReply (
 		    status = IcePaAuthAccepted;
 		}
 
-		if (hostname)
-		    free (hostname);
+		free (hostname);
 	    }
 
 	    if (status == IcePaAuthRejected)
@@ -1517,19 +1566,16 @@ ProcessAuthReply (
 		_IceErrorSetupFailed (iceConn, ICE_ProtocolSetup,
 		    failureReason);
 
-		if (failureReason)
-		    free (failureReason);
+		free (failureReason);
 	    }
 	}
 
 
 	if (free_setup_info)
 	{
-	    if (iceConn->protosetup_to_me->his_vendor)
-		free (iceConn->protosetup_to_me->his_vendor);
-	    if (iceConn->protosetup_to_me->his_release)
-		free (iceConn->protosetup_to_me->his_release);
-	    free ((char *) iceConn->protosetup_to_me);
+	    free (iceConn->protosetup_to_me->his_vendor);
+	    free (iceConn->protosetup_to_me->his_release);
+	    free (iceConn->protosetup_to_me);
 	    iceConn->protosetup_to_me = NULL;
 	}
     }
@@ -1543,17 +1589,17 @@ ProcessAuthReply (
     }
 
     if (authData && authDataLen > 0)
-	free ((char *) authData);
+	free (authData);
 
-    if (errorString)
-	free (errorString);
+
+    free (errorString);
 
     IceDisposeCompleteMessage (iceConn, replyData);
     return (0);
 }
 
 
-
+
 static Bool
 ProcessAuthNextPhase (
 	IceConn		  	iceConn,
@@ -1636,7 +1682,8 @@ ProcessAuthNextPhase (
     }
     else if (status == IcePoAuthRejected || status == IcePoAuthFailed)
     {
-	char *prefix = NULL, *returnErrorString;
+	const char *prefix = NULL;
+	char *returnErrorString;
 
 	if (status == IcePoAuthRejected)
 	{
@@ -1653,9 +1700,8 @@ ProcessAuthNextPhase (
 	    prefix = "Authentication Failed, reason : ";
 	}
 
-	returnErrorString = (char *) malloc (strlen (prefix) +
-	    strlen (errorString) + 1);
-	sprintf (returnErrorString, "%s%s", prefix, errorString);
+	if (asprintf (&returnErrorString, "%s%s", prefix, errorString) == -1)
+	    returnErrorString = strdup("");
 	free (errorString);
 
 	if (iceConn->connect_to_you)
@@ -1677,7 +1723,7 @@ ProcessAuthNextPhase (
     }
 
     if (replyData && replyDataLen > 0)
-	free ((char *) replyData);
+	free (replyData);
 
     IceDisposeCompleteMessage (iceConn, authData);
 
@@ -1685,7 +1731,7 @@ ProcessAuthNextPhase (
 }
 
 
-
+
 static Bool
 ProcessConnectionReply (
 	IceConn			iceConn,
@@ -1752,10 +1798,10 @@ ProcessConnectionReply (
 
 	    _IceErrorBadValue (iceConn, 0,
 		ICE_ConnectionReply, 2, 1, &errIndex);
-	    
+
 	    errorReply->type = ICE_CONNECTION_ERROR;
 	    errorReply->error_message =
-		"Received bad version index in Connection Reply";
+		strdup("Received bad version index in Connection Reply");
 	}
 	else
 	{
@@ -1787,7 +1833,7 @@ ProcessConnectionReply (
 }
 
 
-
+
 static int
 ProcessProtocolSetup (
 	IceConn		iceConn,
@@ -1803,7 +1849,7 @@ ProcessProtocolSetup (
     int	 	      	myAuthCount, hisAuthCount;
     int  	      	myOpcode, hisOpcode;
     int	 	      	found, i, j;
-    char	      	*myAuthName, **hisAuthNames = NULL;
+    char	      	**hisAuthNames = NULL;
     char 	      	*protocolName;
     char 		*pData, *pStart, *pEnd;
     char 	      	*vendor = NULL;
@@ -1843,16 +1889,16 @@ ProcessProtocolSetup (
     pEnd = pStart + (length << 3);
 
     SKIP_STRING (pData, swap, pEnd,
-		 BAIL_STRING(iceConn, ICE_ProtocolSetup, 
+		 BAIL_STRING(iceConn, ICE_ProtocolSetup,
 			     pStart));			       /* proto name */
     SKIP_STRING (pData, swap, pEnd,
-		 BAIL_STRING(iceConn, ICE_ProtocolSetup, 
+		 BAIL_STRING(iceConn, ICE_ProtocolSetup,
 			     pStart));			       /* vendor */
     SKIP_STRING (pData, swap, pEnd,
-		 BAIL_STRING(iceConn, ICE_ProtocolSetup, 
+		 BAIL_STRING(iceConn, ICE_ProtocolSetup,
 			     pStart));			       /* release */
     SKIP_LISTOF_STRING (pData, swap, (int) message->authCount, pEnd,
-			BAIL_STRING(iceConn, ICE_ProtocolSetup, 
+			BAIL_STRING(iceConn, ICE_ProtocolSetup,
 				    pStart));		       /* auth names */
     pData += (message->versionCount * 4);		       /* versions */
 
@@ -1923,9 +1969,10 @@ ProcessProtocolSetup (
     EXTRACT_STRING (pData, swap, vendor);
     EXTRACT_STRING (pData, swap, release);
 
-    if ((hisAuthCount = message->authCount) > 0)
+    hisAuthCount = message->authCount;
+    if (hisAuthCount > 0)
     {
-	hisAuthNames = (char **) malloc (hisAuthCount * sizeof (char *));
+	hisAuthNames = malloc (hisAuthCount * sizeof (char *));
 	EXTRACT_LISTOF_STRING (pData, swap, hisAuthCount, hisAuthNames);
     }
 
@@ -1962,8 +2009,8 @@ ProcessProtocolSetup (
 	{
 	    for (i = 0; i < hisAuthCount; i++)
 		free (hisAuthNames[i]);
-	
-	    free ((char *) hisAuthNames);
+
+	    free (hisAuthNames);
 	}
 
 	IceDisposeCompleteMessage (iceConn, pStart);
@@ -1974,7 +2021,8 @@ ProcessProtocolSetup (
 
     _IceGetPaValidAuthIndices (
 	_IceProtocols[myOpcode - 1].protocol_name,
-	iceConn->connection_string, myAuthCount, myProtocol->auth_names,
+	iceConn->connection_string, myAuthCount,
+	(const char **) myProtocol->auth_names,
         &authUsableCount, authIndices);
 
     for (i = 0; i < myAuthCount; i++)
@@ -1988,7 +2036,7 @@ ProcessProtocolSetup (
     {
 	if (authUsableFlags[i])
 	{
-	    myAuthName = myProtocol->auth_names[i];
+	    const char *myAuthName = myProtocol->auth_names[i];
 
 	    for (j = 0; j < hisAuthCount && !found; j++)
 		if (strcmp (myAuthName, hisAuthNames[j]) == 0)
@@ -2022,14 +2070,13 @@ ProcessProtocolSetup (
 	    {
 		accept_setup_now = 1;
 	    }
-	    else 
+	    else
 	    {
 		_IceErrorAuthenticationRejected (iceConn,
 	            ICE_ProtocolSetup, "None of the authentication protocols specified are supported and host-based authentication failed");
 	    }
 
-	    if (hostname)
-		free (hostname);
+	    free (hostname);
 	}
     }
     else
@@ -2052,10 +2099,9 @@ ProcessProtocolSetup (
 	    _IceProtoSetupToMeInfo *setupInfo;
 
 	    AuthRequired (iceConn, hisAuthIndex, authDataLen, authData);
-	 
+
 	    iceConn->protosetup_to_me = setupInfo =
-		(_IceProtoSetupToMeInfo *) malloc (
-		sizeof (_IceProtoSetupToMeInfo));
+		malloc (sizeof (_IceProtoSetupToMeInfo));
 
 	    setupInfo->his_opcode = hisOpcode;
 	    setupInfo->my_opcode = myOpcode;
@@ -2074,10 +2120,10 @@ ProcessProtocolSetup (
 	}
 
 	if (authData && authDataLen > 0)
-	    free ((char *) authData);
+	    free (authData);
 
-	if (errorString)
-	    free (errorString);
+
+	free (errorString);
     }
 
     if (accept_setup_now)
@@ -2160,23 +2206,20 @@ ProcessProtocolSetup (
 
 	    _IceErrorSetupFailed (iceConn, ICE_ProtocolSetup, failureReason);
 
-	    if (failureReason)
-		free (failureReason);
+	    free (failureReason);
 	}
     }
 
-    if (vendor)
-	free (vendor);
 
-    if (release)
-	free (release);
+    free (vendor);
+    free (release);
 
     if (hisAuthCount > 0)
     {
 	for (i = 0; i < hisAuthCount; i++)
 	    free (hisAuthNames[i]);
 
-	free ((char *) hisAuthNames);
+	free (hisAuthNames);
     }
 
     IceDisposeCompleteMessage (iceConn, pStart);
@@ -2184,7 +2227,7 @@ ProcessProtocolSetup (
 }
 
 
-
+
 static Bool
 ProcessProtocolReply (
 	IceConn		  	iceConn,
@@ -2241,17 +2284,7 @@ ProcessProtocolReply (
 	    IcePoAuthProc authProc = myProtocol->auth_procs[(int)
 		(iceConn->protosetup_to_you->my_auth_index)];
 
-#ifdef SVR4
-
-/*
- * authProc is never NULL, but the cc compiler on UNIX System V/386
- * Release 4.2 Version 1 screws up an optimization.  Unless there is
- * some sort of reference to authProc before the function call, the
- * function call will seg fault.
- */
-	    if (authProc)
-#endif
-		(*authProc) (iceConn,
+	    (*authProc) (iceConn,
 		&iceConn->protosetup_to_you->my_auth_state,
 		True /* clean up */, False /* swap */,
 	        0, NULL, NULL, NULL, NULL);
@@ -2265,14 +2298,14 @@ ProcessProtocolReply (
 
 	    _IceErrorBadValue (iceConn, 0,
 		ICE_ProtocolReply, 2, 1, &errIndex);
-	    
+
 	    errorReply->type = ICE_PROTOCOL_ERROR;
 	    errorReply->error_message =
-		"Received bad version index in Protocol Reply";
+		strdup("Received bad version index in Protocol Reply");
 	}
 	else
 	{
-	    _IceProtocolReply *reply = 
+	    _IceProtocolReply *reply =
 	        &(((_IceReply *) (replyWait->reply))->protocol_reply);
 
 	    reply->type = ICE_PROTOCOL_REPLY;
@@ -2298,7 +2331,7 @@ ProcessProtocolReply (
 }
 
 
-
+
 static int
 ProcessPing (
 	IceConn 	iceConn,
@@ -2314,7 +2347,7 @@ ProcessPing (
 }
 
 
-
+
 static int
 ProcessPingReply (
 	IceConn 	iceConn,
@@ -2327,11 +2360,11 @@ ProcessPingReply (
     if (iceConn->ping_waits)
     {
 	_IcePingWait *next = iceConn->ping_waits->next;
-	
+
 	(*iceConn->ping_waits->ping_reply_proc) (iceConn,
 	    iceConn->ping_waits->client_data);
 
-	free ((char *) iceConn->ping_waits);
+	free (iceConn->ping_waits);
 	iceConn->ping_waits = next;
     }
     else
@@ -2343,7 +2376,7 @@ ProcessPingReply (
 }
 
 
-
+
 static int
 ProcessWantToClose (
 	IceConn 	iceConn,
@@ -2402,7 +2435,7 @@ ProcessWantToClose (
 }
 
 
-
+
 static int
 ProcessNoClose (
 	IceConn 	iceConn,
@@ -2430,7 +2463,7 @@ ProcessNoClose (
 }
 
 
-
+
 static void
 _IceProcessCoreMessage (
 	IceConn 	 iceConn,
@@ -2519,7 +2552,7 @@ _IceProcessCoreMessage (
 	*replyReadyRet = replyReady;
 }
 
-int		_IceVersionCount = 1;
-_IceVersion	_IceVersions[] = {
-		    {IceProtoMajor, IceProtoMinor, _IceProcessCoreMessage}};
+const int		_IceVersionCount = 1;
+const _IceVersion	_IceVersions[] = {
+			    {IceProtoMajor, IceProtoMinor, _IceProcessCoreMessage}};
 
