@@ -71,7 +71,10 @@ IceProtocolSetup (
 
     if (myOpcode < 1 || myOpcode > _IceLastMajorOpcode)
     {
-	strncpy (errorStringRet, "myOpcode out of range", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "myOpcode out of range", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (IceProtocolSetupFailure);
     }
 
@@ -79,8 +82,11 @@ IceProtocolSetup (
 
     if (myProtocol->orig_client == NULL)
     {
-	strncpy (errorStringRet,
-	    "IceRegisterForProtocolSetup was not called", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet,
+		"IceRegisterForProtocolSetup was not called", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (IceProtocolSetupFailure);
     }
 
@@ -112,7 +118,7 @@ IceProtocolSetup (
 
     if (myProtocol->orig_client->auth_count > 0)
     {
-	authIndices = (int *) malloc (
+	authIndices = malloc (
 	    myProtocol->orig_client->auth_count * sizeof (int));
 
 	_IceGetPoValidAuthIndices (myProtocol->protocol_name,
@@ -181,8 +187,7 @@ IceProtocolSetup (
     replyWait.minor_opcode_of_request = ICE_ProtocolSetup;
     replyWait.reply = (IcePointer) &reply;
 
-    iceConn->protosetup_to_you = (_IceProtoSetupToYouInfo *) malloc (
-	sizeof (_IceProtoSetupToYouInfo));
+    iceConn->protosetup_to_you = malloc (sizeof (_IceProtoSetupToYouInfo));
     iceConn->protosetup_to_you->my_opcode = myOpcode;
     iceConn->protosetup_to_you->my_auth_count = authCount;
     iceConn->protosetup_to_you->auth_active = 0;
@@ -199,9 +204,12 @@ IceProtocolSetup (
 
 	if (ioErrorOccured)
 	{
-	    strncpy (errorStringRet,
-		"IO error occured doing Protocol Setup on connection",
-		errorLength);
+	    if (errorStringRet && errorLength > 0) {
+		strncpy (errorStringRet,
+		    "IO error occured doing Protocol Setup on connection",
+		    errorLength);
+		errorStringRet[errorLength - 1] = '\0';
+	    }
 	    return (IceProtocolSetupIOError);
 	}
 	else if (gotReply)
@@ -211,9 +219,12 @@ IceProtocolSetup (
 		if (reply.protocol_reply.version_index >=
 		    myProtocol->orig_client->version_count)
 		{
-		    strncpy (errorStringRet,
-	                "Got a bad version index in the Protocol Reply",
-		        errorLength);
+		    if (errorStringRet && errorLength > 0) {
+			strncpy (errorStringRet,
+	                    "Got a bad version index in the Protocol Reply",
+		            errorLength);
+			errorStringRet[errorLength - 1] = '\0';
+		    }
 
 		    free (reply.protocol_reply.vendor);
 		    free (reply.protocol_reply.release);
@@ -230,15 +241,18 @@ IceProtocolSetup (
 	    {
 		/* Protocol Setup failed */
 
-		strncpy (errorStringRet, reply.protocol_error.error_message,
-		    errorLength);
+		if (errorStringRet && errorLength > 0) {
+		    strncpy (errorStringRet, reply.protocol_error.error_message,
+			errorLength);
+		    errorStringRet[errorLength - 1] = '\0';
+		}
 
 		free (reply.protocol_error.error_message);
 	    }
 
 	    if (iceConn->protosetup_to_you->my_auth_indices)
-		free ((char *) iceConn->protosetup_to_you->my_auth_indices);
-	    free ((char *) iceConn->protosetup_to_you);
+		free (iceConn->protosetup_to_you->my_auth_indices);
+	    free (iceConn->protosetup_to_you);
 	    iceConn->protosetup_to_you = NULL;
 	}
     }
